//Part of the prototypical solution for the Microcontroller Programming Laboratory
//term project, winter semester 2010/11
// (c) Andreas Unterweger, Fachhochschule Salzburg, ITS

#include <at8xc51snd1.h>
#include "tacho.h"
#include "pwm.h"
#include "lcd.h"
#include "adc.h"
#include "leds.h"
#include "utils.h"
#include "data.h"

//Main function
void main()
{
  unsigned short rpm;
  unsigned short overflow = 0;
  unsigned char run = RunningLightStart; //Running light
  DisableX2Mode();
  InitPWM();
  InitTacho();
  InitLCD();
  InitADC();
  InitLEDs();
  while (1) //Main loop
  {
    if (overflow++ == 0) //Only refresh on overflow (every 65.536th iteration) to avoid LCD flickering
    {
	  rpm = GetLastTachoReadingRPM();
      ClearLCD();
      WriteLCDChars(FANSPEED_TEXT, FANSPEED_TEXT_LEN);
      if (!rpm) //Show "no input" message until there is a valid RPM value
        WriteLCDChars(NO_INPUT_MSG, NO_INPUT_MSG_LEN);
      else //Display RPM value
      {
        SpacePaddedDecimalToASCII(rpm, rpm_str, RPM_STR_LEN);
        WriteLCDChars(rpm_str, RPM_STR_LEN);
        WriteLCDChars(RPM_TEXT, RPM_TEXT_LEN);
      }
      WriteLCDChars(AIMED_DUTY_TEXT, AIMED_DUTY_TEXT_LEN);
      rpm = ReadADCChannel(1); //Get desired duty as byte value
      SetHigherLEDRow(ByteToProgress(rpm));
      SetLowerLEDRow(NextRunningLight(run)); //Update running light
      rpm = ByteToPercent(rpm);
      SetPWMDuty(rpm); //Set duty
      SpacePaddedDecimalToASCII(rpm, rpm_str, RPM_STR_LEN);
      WriteLCDChars(rpm_str, RPM_STR_LEN);
      WriteLCDChar('%');
    }
  }
}